Index: src/nsca.c
===================================================================
RCS file: /cvsroot/nagios/nsca/src/nsca.c,v
retrieving revision 1.26
diff -u -r1.26 nsca.c
--- src/nsca.c	15 Jan 2008 22:26:00 -0000	1.26
+++ src/nsca.c	16 Dec 2008 14:44:58 -0000
@@ -815,13 +815,45 @@
         }
 
 
+static int get_peer_info(int sock, char **peer_ip, int *peer_port)
+{
+  struct sockaddr addr;
+  struct sockaddr_in *nptr;
+  socklen_t addrlen;
+  int rc;
+
+  /* Initialize to null values */
+  if (peer_ip)
+    *peer_ip = 0;
+  if (peer_port)
+    *peer_port = 0;
+
+  addrlen = sizeof(addr);
+  rc = getpeername(sock, &addr, &addrlen);
+  if (rc < 0) {
+    return rc;
+  }
+
+  nptr = (struct sockaddr_in *)&addr;
+
+  if (peer_ip) {
+    if (! (*peer_ip = strdup(inet_ntoa(nptr->sin_addr)))) {
+      return -1;
+    }
+  }
+  
+  if (peer_port) {
+    *peer_port = ntohs(nptr->sin_port);
+  }
+
+  return 0;
+}
 
 static void accept_connection(int sock, void *unused){
         int new_sd;
         pid_t pid;
-        struct sockaddr addr;
-        struct sockaddr_in *nptr;
-        socklen_t addrlen;
+	char *peer_ip;
+	int peer_port;
         int rc;
 #ifdef HAVE_LIBWRAP
 	struct request_info req;
@@ -901,8 +933,7 @@
                 }
 
         /* find out who just connected... */
-        addrlen=sizeof(addr);
-        rc=getpeername(new_sd,&addr,&addrlen);
+        rc=get_peer_info(new_sd,&peer_ip,&peer_port);
 
         if(rc<0){
                 /* log error to syslog facility */
@@ -915,11 +946,11 @@
 		return;
                 }
 
-        nptr=(struct sockaddr_in *)&addr;
-
         /* log info to syslog facility */
         if(debug==TRUE)
-                syslog(LOG_DEBUG,"Connection from %s port %d",inet_ntoa(nptr->sin_addr),nptr->sin_port);
+	  syslog(LOG_DEBUG,"Connection from %s port %d",peer_ip, peer_port);
+
+	free(peer_ip);
 
 	/* handle the connection */
 	if(mode==SINGLE_PROCESS_DAEMON)
@@ -934,6 +965,9 @@
 
 
 
+#define GET_PEER_IP { save_errno = errno; (void) get_peer_info(sock, &peer_ip, 0); if (! peer_ip) peer_ip = strdup("UNKNOWN"); errno = save_errno; }
+#define USE_GET_PEER_IP char *peer_ip; int save_errno;
+
 /* handle a client connection */
 static void handle_connection(int sock, void *data){
         init_packet send_packet;
@@ -942,7 +976,11 @@
         int flags;
         time_t packet_send_time;
         struct crypt_instance *CI;
+	USE_GET_PEER_IP;
 
+	/* Do this right away, in case we lose the connection and can
+	   no longer get the IP to log it. */
+	GET_PEER_IP;
 
         /* log info to syslog facility */
         if(debug==TRUE)
@@ -957,6 +995,7 @@
                 close(sock);
 		if(mode==MULTI_PROCESS_DAEMON)
 			do_exit(STATE_CRITICAL);
+		free(peer_ip);
                 return;
                 }
 
@@ -971,7 +1010,8 @@
 
         /* there was an error sending the packet */
         if(rc==-1){
-                syslog(LOG_ERR,"Could not send init packet to client\n");
+                syslog(LOG_ERR,"Could not send init packet to client %s: %m", peer_ip);
+		free(peer_ip);
                 encrypt_cleanup(decryption_method,CI);
                 close(sock);
 		if(mode==MULTI_PROCESS_DAEMON)
@@ -981,7 +1021,8 @@
 
         /* for some reason we didn't send all the bytes we were supposed to */
 	else if(bytes_to_send<sizeof(send_packet)){
-                syslog(LOG_ERR,"Only able to send %d of %d bytes of init packet to client\n",rc,sizeof(send_packet));
+                syslog(LOG_ERR,"Only able to send %d of %d bytes of init packet to client %s: %m",rc,sizeof(send_packet),peer_ip);
+		free(peer_ip);
                 encrypt_cleanup(decryption_method,CI);
                 close(sock);
 		if(mode==MULTI_PROCESS_DAEMON)
@@ -989,6 +1030,8 @@
                 return;
                 }
 
+	free(peer_ip);
+
         /* open the command file if we're aggregating writes */
         if(aggregate_writes==TRUE){
                 if(open_command_file()==ERROR){
@@ -1026,6 +1069,11 @@
         char host_name[MAX_HOSTNAME_LENGTH];
         char svc_description[MAX_DESCRIPTION_LENGTH];
         char plugin_output[MAX_PLUGINOUTPUT_LENGTH];
+	USE_GET_PEER_IP;
+
+	/* Do this right away, in case we lose the connection and can
+	   no longer get the IP to log it. */
+	GET_PEER_IP;
 
         CI=data;
 
@@ -1037,7 +1085,12 @@
 
         /* recv() error or client disconnect */
         if(rc<=0){
-                if(debug==TRUE)
+		if (rc < 0) {
+		  syslog(LOG_ERR, "Error reading from client %s: %s", peer_ip,
+			 (errno == EAGAIN) ? "Timeout" : strerror(errno));
+		  free(peer_ip);
+		}
+                else if(debug==TRUE)
                         syslog(LOG_ERR,"End of connection...");
                 encrypt_cleanup(decryption_method, CI);
                 close(sock);
@@ -1049,7 +1102,8 @@
 
         /* we couldn't read the correct amount of data, so bail out */
         if(bytes_to_recv!=sizeof(receive_packet)){
-                syslog(LOG_ERR,"Data sent from client was too short (%d < %d), aborting...",bytes_to_recv,sizeof(receive_packet));
+                syslog(LOG_ERR,"Data sent from client %s was too short (%d < %d), aborting...",peer_ip,bytes_to_recv,sizeof(receive_packet));
+		free(peer_ip);
                 encrypt_cleanup(decryption_method, CI);
                 close(sock);
 		return;
@@ -1068,7 +1122,8 @@
 
         /* make sure this is the right type of packet */
         if(ntohs(receive_packet.packet_version)!=NSCA_PACKET_VERSION_3){
-                syslog(LOG_ERR,"Received invalid packet type/version from client - possibly due to client using wrong password or crypto algorithm?");
+                syslog(LOG_ERR,"Received invalid packet type/version from client %s - possibly due to client using wrong password or crypto algorithm?",peer_ip);
+		free(peer_ip);
 		/*return;*/
 		close(sock);
                 if(mode==SINGLE_PROCESS_DAEMON)
@@ -1082,7 +1137,8 @@
         receive_packet.crc32_value=0L;
         calculated_crc32=calculate_crc32((char *)&receive_packet,sizeof(receive_packet));
         if(packet_crc32!=calculated_crc32){
-                syslog(LOG_ERR,"Dropping packet with invalid CRC32 - possibly due to client using wrong password or crypto algorithm?");
+                syslog(LOG_ERR,"Dropping packet with invalid CRC32 from %s - possibly due to client using wrong password or crypto algorithm?",peer_ip);
+		free(peer_ip);
                 /*return;*/
 		close(sock);
                 if(mode==SINGLE_PROCESS_DAEMON)
@@ -1095,7 +1151,8 @@
         packet_time=(time_t)ntohl(receive_packet.timestamp);
         time(&current_time);
         if(packet_time>current_time){
-                syslog(LOG_ERR,"Dropping packet with future timestamp.");
+                syslog(LOG_ERR,"Dropping packet with future timestamp from %s.",peer_ip);
+		free(peer_ip);
                 /*return;*/
 		close(sock);
                 if(mode==SINGLE_PROCESS_DAEMON)
@@ -1106,7 +1163,8 @@
 	else{
                 packet_age=(unsigned long)(current_time-packet_time);
                 if(max_packet_age>0 && (packet_age>max_packet_age)){
-                        syslog(LOG_ERR,"Dropping packet with stale timestamp - packet was %lu seconds old.",packet_age);
+                        syslog(LOG_ERR,"Dropping packet with stale timestamp from %s - packet was %lu seconds old.",peer_ip,packet_age);
+			free(peer_ip);
                         /*return;*/
 			close(sock);
 			if(mode==SINGLE_PROCESS_DAEMON)
@@ -1116,6 +1174,8 @@
                         }
                 }
 
+	free(peer_ip);
+
         /**** GET THE SERVICE CHECK INFORMATION ****/
 
         /* plugin return code */
